package org.ibex.mail;

import org.ibex.io.*;
import org.ibex.mail.protocol.*;
import org.ibex.util.*;
import org.ibex.net.*;
import java.sql.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.Timestamp;
import java.sql.Connection;

public class SqliteTable {

    protected Connection conn;
    private String filename;
    private String reapTable;
    private String reapColumn;

    public SqliteTable(String filename, String[] tables, String reapTable, String reapColumn) {
        this(filename, tables, false, reapTable, reapColumn);
    }
    public SqliteTable(String filename, String[] tables, boolean fastButDangerous,
                       String reapTable, String reapColumn) {
        this.filename = filename;
        try {
            Class.forName("org.sqlite.JDBC");
            conn = DriverManager.getConnection("jdbc:sqlite:"+filename);
            for(String s : tables)
                conn.prepareStatement(s).executeUpdate();
            conn.prepareStatement("PRAGMA temp_store = MEMORY").executeUpdate();
            conn.prepareStatement("PRAGMA page_size=4096").executeUpdate();
            conn.prepareStatement("PRAGMA cache_size=2000").executeUpdate();
            if (fastButDangerous)
                conn.prepareStatement("PRAGMA synchronous = OFF").executeUpdate();
        }
        catch (SQLException e) { throw new RuntimeException(e); }
        catch (ClassNotFoundException e) { throw new RuntimeException(e); }
        this.reapTable = reapTable;
        this.reapColumn = reapColumn;
        if (reapTable != null && reapColumn != null)
            new Reaper().start();
    }

    public static final int REAPER_INTERVAL_SECONDS = 60 * 60;

    private class Reaper extends Thread {

        public void run() {
            while(true) {
                try {
                    try { Thread.sleep(1000 * REAPER_INTERVAL_SECONDS); } catch (Exception e) { };
                    Log.warn(Reaper.class, filename + " reaping...");
                    long when = System.currentTimeMillis();
                    when -= 5 * 24 * 60 * 60 * 1000;
                    synchronized(SqliteTable.this) {
                        PreparedStatement ps = conn.prepareStatement("select count(*) from "+reapTable+" where "+reapColumn+"<?");
                        ps.setTimestamp(1, new Timestamp(when));
                        ResultSet rs = ps.executeQuery();
                        if (rs.next())
                            Log.warn(Reaper.class, filename + " reaping " + rs.getInt(1) + " entries");
                        Log.warn(Reaper.class, filename + ": " + "delete from "+reapTable+" where "+reapColumn+"<"+when);
                        ps = conn.prepareStatement("delete from "+reapTable+" where "+reapColumn+"<?");
                        ps.setTimestamp(1, new Timestamp(when));
                        int rows = ps.executeUpdate();
                        Log.warn(Reaper.class, filename + " done reaping; removed " + rows + " rows");
                    }
                } catch (Exception e) { Log.error(Reaper.class, e); }
            }
        }
    }

}
